/*
 * Simple Jitter program (client side)
 * Alcatel Alenia Space (c) 2006
 * Author: Nicolas Hennion
 */
 
#include "jitter.h"

#define DEFAULT_PACKET_NUMBER 	10

int socketfd; /* Global variable for the socket file descriptor*/

int main(int argc, char **argv) {
	struct sockaddr_in serveraddr;
	socklen_t slen=sizeof(serveraddr);
	char buf[BUFFER_SIZE], server_ip[15];
	extern char *optarg;
	int i, errflag=1;
	int packet_number=DEFAULT_PACKET_NUMBER, port_number=DEFAULT_PORT_NUMBER, bitrate=DEFAULT_BITRATE;
	double begin=0, end=0, sendbegin=0, sendend=0;
	unsigned long packet_delay, packet_size;
	
	/* Manage arguments */
	while ((i = getopt(argc, argv, "c:n:p:b:")) != EOF) { 
		switch (i) { 
			case 'c': 
				strncpy(server_ip, optarg, 15);
				errflag = 0;
				break; 
			case 'n':
				sscanf(optarg, "%d", &packet_number);
				if ((packet_number < 2) || (packet_number > MAX_PACKET_NUMBER))
					errflag = 1;
				break;
			case 'p':
				sscanf(optarg, "%d", &port_number);
				if ((port_number < 1024) || (port_number > MAX_PORT_NUMBER))
					errflag = 1;
				break;
			case 'b':
				sscanf(optarg, "%d", &bitrate);
				if (bitrate < 11 )
					errflag = 1;
				break;
		} 
	} 
	if (errflag) { 
		fprintf(stderr, "usage: sjitterc -c ADDR [-n PN] [-p PORT]\n"); 
		fprintf(stderr, "\t-c SERVER_IP_ADDRESS: where ADDR is the server IP address\n");
		fprintf(stderr, "\t-n PN: where PN is the number of datagram (>1 , <%d) [default:%d]\n", MAX_PACKET_NUMBER, DEFAULT_PACKET_NUMBER);
		fprintf(stderr, "\t-p PORT: where PORT is the port number (>1024, <%d) [default:%d]\n", MAX_PORT_NUMBER, DEFAULT_PORT_NUMBER);
		fprintf(stderr, "\t-b BR: where BR is the bitrate in Kbps (>10) [default:%d]\n", DEFAULT_BITRATE);	
		exit(2);
	}

	/* Signals management */
	signal(SIGINT,onsignal);
	signal(SIGTERM,onsignal);
	signal(SIGQUIT,onsignal);	

	/* Build the socket */
	if ((socketfd=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1)
		errorexit("socket");

	memset((char *) &serveraddr, sizeof(serveraddr), 0);
	serveraddr.sin_family = AF_INET;
	serveraddr.sin_port = htons(port_number);
	if (inet_aton(server_ip, &serveraddr.sin_addr)==0)
		perror("inet_aton");
	
	printf("Send data (%d datagrams / %d Kbps) to the server:port %s:%d\n", 
			packet_number, bitrate, inet_ntoa(serveraddr.sin_addr), port_number);
	
	/* Send configuration datagram to the server */
	sprintf(buf, "SJITTER-START %d ", packet_number);
	if (sendto(socketfd, buf, BUFFER_SIZE, 0, (const struct sockaddr *) &serveraddr, slen)==-1)
			errorexit("sendto");

	/* Compute delay for bitrate (goal on the IP level) */
	packet_size = (unsigned long) ((sizeof(buf)+28)*8);
	packet_delay = (unsigned long) ((packet_size/(bitrate*1000.0))*1000000);
	/* printf("packet_size=%ld / packet_delay=%ld\n", packet_size, packet_delay); */
	
	/* Send data to the server */
	printf("Sending data...\n");
	begin = getcurrenttimems();
	for (i=0; i<packet_number; i++) {
		
		/* Manage bitrate */
		usleep(packet_delay-((sendend-sendbegin)*1000000));
		
		/* Send datagram on the network interface */
		sendbegin = getcurrenttimems();	/* Send begin... */		
		sprintf(buf, "SJITTER-DATA %d %lf ", i, (double) getcurrenttimems());
		if (sendto(socketfd, buf, sizeof(buf), 0, (const struct sockaddr *) &serveraddr, slen)==-1)
			errorexit("sendto");
		progressbar(i, packet_number);		
		sendend = getcurrenttimems(); /* ... send end */
	}
	end = getcurrenttimems();

	/* Send "end" datagram to the server */
	sprintf(buf, "SJITTER-END ");
	if (sendto(socketfd, buf, BUFFER_SIZE, 0, (const struct sockaddr *) &serveraddr, slen)==-1)
			errorexit("sendto");
		
	/* Summary */
	printf("\r                                                                                \r");
	printf("Summary: %d datagrams sent in %.2lf seconds (%.0lf Kbps)\n", 
		   packet_number, end-begin, (packet_number*sizeof(buf)*8)/(end-begin)/1000);
	
	close(socketfd);
	return 0;
}
