/*
 * Simple Jitter program (server side)
 * Alcatel Alenia Space (c) 2006
 * Author: Nicolas Hennion
 */
 
#include "jitter.h"

int socketfd; /* Global variable for the socket file descriptor*/

int main(int argc, char **argv) {
	struct sockaddr_in serveraddr, clientaddr;
	socklen_t slen=sizeof(clientaddr);
	char buf[BUFFER_SIZE], buf2[BUFFER_SIZE];
	extern char *optarg;
	int i, pn, pid;
	int packet_number, port_number=DEFAULT_PORT_NUMBER;
	int errflag=0, iflag=0, checkconf=0, endflag=0;
	double begin=0, end=0, t_send, t_receive, t_transit, t_transitlast, t_transitdelta;
	double jitterlast=0, jittermin=0, jittermax=0, jitteravg=0, jittersum=0, jitterlist[MAX_PACKET_NUMBER];	
	double onewaydelaylast=0, onewaydelaymin=0, onewaydelaymax=0, onewaydelayavg=0, onewaydelaysum=0, onewaydelaylist[MAX_PACKET_NUMBER];	

	/* Manage arguments */
	while ((i = getopt(argc, argv, "ip:")) != EOF) { 
		switch (i) { 
			case 'i':
				iflag++;
				break;
			case 'p':
				sscanf(optarg, "%d", &port_number);
				if ((port_number < 1024) || (port_number > MAX_PORT_NUMBER))
					errflag = 1;
				break;
		} 
	} 
	if (errflag) { 
		fprintf(stderr, "usage: sjitters [-p PORT]\n"); 
		fprintf(stderr, "\t-p PORT: where PORT is the port number (>1024, <%d) [default:%d]\n", MAX_PORT_NUMBER, DEFAULT_PORT_NUMBER);		
		exit(2);
	}

	/* Signals management */
	signal(SIGINT,onsignal);
	signal(SIGTERM,onsignal);
	signal(SIGQUIT,onsignal);	

	/* Build the socket */
	if ((socketfd=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1)
		errorexit("socket");	
	memset((char *) &serveraddr, sizeof(serveraddr), 0);
	serveraddr.sin_family = AF_INET;
	serveraddr.sin_port = htons(port_number);
	serveraddr.sin_addr.s_addr = htonl(INADDR_ANY);
	if (bind(socketfd, (struct sockaddr *) &serveraddr, sizeof(serveraddr))==-1)
		errorexit("bind");

	/* Main loop */
	while (1) {
		
		/* Wait START (configuration) datagram */
		while (checkconf==0) {
			if (recvfrom(socketfd, buf, BUFFER_SIZE, 0, (struct sockaddr *) &clientaddr, &slen)==-1)
				errorexit("recvfrom");
			if ((sscanf(buf, "%s %d ", (char *) &buf2, &packet_number) == 2)
				&& (strncmp(buf2, "SJITTER-START", BUFFER_SIZE) == 0) 
				&& (packet_number > 0)) {
					checkconf = 1;
				}
		}
		
		/* 	A START datagram has been received */
		if ((pid = fork()) < 0) {
			errorexit("fork");
		}
		if (pid > 0) {
			/* Father process */
			
			/* Wait the end of the child process */
			waitpid(pid, NULL, 0);
			
			/* Wait for another START datagram */		
			checkconf = 0;
	
		} else { /* pid == 0 */
			/* Child process */
			printf("[Client: %s] Init OK (wait %d datagrams)\n", inet_ntoa(clientaddr.sin_addr), packet_number);
			
			/* Wait data */
			i = 0;
			begin = getcurrenttimems();
			while (!endflag) {
				if (recvfrom(socketfd, buf, BUFFER_SIZE, 0, (struct sockaddr *) &clientaddr, &slen)==-1)
					errorexit("recvfrom");
		
				/* Compute Jitter (see in RFC 1889) */
				t_receive = getcurrenttimems();
				if (sscanf (buf, "%s %d %lf ", (char *) &buf2, &pn, &t_send) > 0) {
					if (strncmp(buf2, "SJITTER-DATA", BUFFER_SIZE) == 0) {
						/* SJITTER-DATA: the server has received a DATA packet */
						t_transit = t_receive-t_send;
						if (i != 0) {
							t_transitdelta = t_transit - t_transitlast;
							if (t_transitdelta < 0.0)
								t_transitdelta = -t_transitdelta;
							
							/* One way delay */
							onewaydelaylast = t_transit;
							if (onewaydelaylast < onewaydelaymax) 
								onewaydelaymin=onewaydelaylast;
							if (onewaydelaylast > onewaydelaymax) 
								onewaydelaymax=onewaydelaylast;
							onewaydelaysum += onewaydelaylast;	
							onewaydelayavg = onewaydelaysum/i;
							onewaydelaylist[i] = onewaydelaylast;							
														
							/* Jitter */
							jitterlast += (t_transitdelta - jitterlast) / (16.0);
							if (jitterlast < jittermax) 
								jittermin=jitterlast;
							if (jitterlast > jittermax) 
								jittermax=jitterlast;
							jittersum += jitterlast;	
							jitteravg = jittersum/i;
							jitterlist[i] = jitterlast;
							
							if (iflag != 0) {
								/* Only display packets if iglag is set (-i option) */
								printf("[Client: %s] ", inet_ntoa(clientaddr.sin_addr));
								printf("One-way delay (ms): Last=%.2lf / Min=%.2lf / Max=%.2lf / Avg=%.2lf\n", onewaydelaylast*1000, onewaydelaymin*1000, onewaydelaymax*1000, onewaydelayavg*1000);
								printf("[Client: %s] ", inet_ntoa(clientaddr.sin_addr));
								printf("Jitter (ms)       : Last=%.2lf / Min=%.2lf / Max=%.2lf / Avg=%.2lf\n", jitterlast*1000, jittermin*1000, jittermax*1000, jitteravg*1000);
							}
						} else {			
							if (iflag != 0) {
								printf("[Client: %s] First datagram received\n", inet_ntoa(clientaddr.sin_addr));
							}
						}
						i++;
					} else if (strncmp(buf2, "SJITTER-END", BUFFER_SIZE) == 0) {
						/* SJITTER-END : the server has received a END packet */
						endflag = 1;
					} else {
						errorexit("Receive an invalid datagram (invalid command)");
					}
				} else {
					errorexit("Receive an invalid datagram (missing parameters))");
				}
				t_transitlast = t_transit;
			}
			end = getcurrenttimems();
							
			/* Display summary */
			printf("[Client: %s] ", inet_ntoa(clientaddr.sin_addr));
			printf("Receive %d/%d (%d%%) datagrams in %.2lf seconds (%.0lf Kbps)\n", 
				   i, packet_number, (i*100)/packet_number, end-begin, (packet_number*sizeof(buf)*8)/(end-begin)/1000);
			printf("[Client: %s] ", inet_ntoa(clientaddr.sin_addr));		   
			printf("One-way delay summary (ms): Min=%.2lf / Max=%.2lf / Avg=%.2lf\n", onewaydelaymin*1000, onewaydelaymax*1000, onewaydelayavg*1000);
			printf("[Client: %s] ", inet_ntoa(clientaddr.sin_addr));		   
			printf("Jitter summary (ms)       : Min=%.2lf / Max=%.2lf / Avg=%.2lf\n", jittermin*1000, jittermax*1000, jitteravg*1000);
						
			exit(0); /* end of the child */
		}

	} /* ens main loop */
	
	/* Close the socket */
	close(socketfd);
	
	return 0; /* end of the father */

}
